@@ -14,7 +14,7 @@ Dancer::Plugin::Database - easy database connections for Dancer applications
 
 =cut
 
-our $VERSION = '1.81';
+our $VERSION = '1.82';
 
 my $settings = undef;
 
@@ -94,6 +94,18 @@ register database => sub {
         if ($handle->{dbh} = _get_connection($conn_details)) {
             $handle->{last_connection_check} = time;
             $handles{$pid_tid}{$handle_key} = $handle;
+
+            if (ref $handle_key && ref $handle_key ne ref $def_handle) {
+                # We were given a hashref of connection settings.  Shove a
+                # reference to that hashref into the handle, so that the hashref
+                # doesn't go out of scope for the life of the handle.
+                # Otherwise, that area of memory could be re-used, and, given
+                # different DB settings in a hashref that just happens to have
+                # the same address, we'll happily hand back the original handle.
+                # See http://bugs.debian.org/cgi-bin/bugreport.cgi?bug=665221
+                # Thanks to Sam Kington for suggesting this fix :)
+                $handle->{_orig_settings_hashref} = $handle_key;
+            }
             return $handle->{dbh};
         } else {
             return;
@@ -194,7 +206,10 @@ sub _get_connection {
         );
         return;
     } elsif (exists $settings->{on_connect_do}) {
-        for (@{ $settings->{on_connect_do} }) {
+        my $to_do = ref $settings->{on_connect_do} eq 'ARRAY'
+            ?   $settings->{on_connect_do}
+            : [ $settings->{on_connect_do} ];
+        for (@$to_do) {
             $dbh->do($_) or Dancer::Logger::error(
                 "Failed to perform on-connect command $_"
             );
@@ -379,7 +394,19 @@ documentation for these.
 
 The optional C<on_connect_do> setting is an array of queries which should be
 performed when a connection is established; if given, each query will be
-performed using C<< $dbh->do >>.
+performed using C<< $dbh->do >>.  (If using MySQL, you might want to use this to
+set C<SQL_MODE> to a suitable value to disable MySQL's built-in free data loss
+'features', for example:
+
+  on_connect_do: "SET SQL_MODE='TRADITIONAL'"
+
+(If you're not familiar with what I mean, I'm talking about the insane default
+behaviour of "hmm, this bit of data won't fit the column you're trying to put it
+in.. hmm, I know, I'll just munge it to fit, and throw a warning afterwards -
+it's not like you're relying on me to, y'know, store what you ask me to store".
+See L<http://effectivemysql.com/presentation/mysql-idiosyncrasies-that-bite/> for
+just one illustration.  In hindsight, I wish I'd made a sensible C<sql_mode> a
+default setting, but I don't want to change that now.)
 
 The optional C<log_queries> setting enables logging of queries generated by the
 helper functions C<quick_insert> et al in L<Dancer::Plugin::Database::Handle>.
